@@ -11,8 +11,17 @@ VENUS_TEST_USE_SSL - Whether to use SSL for the connection. Default False
 
 import os
 import pytest
+import traceback
+import asyncio
 
+orig_close = asyncio.BaseEventLoop.close
 
+def debug_close(self):
+    print("Event loop is being closed. Call stack:")
+    #traceback.print_stack()
+    orig_close(self)
+
+asyncio.BaseEventLoop.close = debug_close
 
 @pytest.fixture
 def config_host():
@@ -26,13 +35,16 @@ def config_port():
 
 @pytest.fixture
 def config_username():
-    return os.getenv("VENUS_TEST_USERNAME", None)
+    return os.getenv("VICTRON_TEST_USERNAME", None)
 
 
 @pytest.fixture
 def config_password():
-    return os.getenv("VENUS_TEST_PASSWORD", None)
+    return os.getenv("VICTRON_TEST_PASSWORD", None)
 
+@pytest.fixture
+def config_root_prefix():
+    return os.getenv("VICTRON_TEST_ROOT_PREFIX", None)
 
 @pytest.fixture
 def config_use_ssl() -> bool:
