@@ -7,7 +7,7 @@ from .device import Device
 from .hub import Hub, CannotConnectError, ProgrammingError, NotConnectedError
 from .metric import Metric
 from .switch import Switch
-from ._victron_enums import DeviceType, InverterMode, GenericOnOff, EvChargerMode
+from ._victron_enums import DeviceType, InverterState, InverterMode, GenericOnOff, EvChargerMode, InverterOverloadAlarmEnum, FluidType, TemperatureType, TemperatureStatus, SolarChargerState, ESSMode, MultiState
 
 __all__ = [
     "Hub",
@@ -26,4 +26,12 @@ __all__ = [
     "EvChargerMode",
     "MetricKind",
     "RangeType",
+    "InverterOverloadAlarmEnum",
+    "InverterState",
+    "TemperatureStatus",
+    "TemperatureType",
+    "FluidType",
+    "SolarChargerState",
+    "ESSMode",
+    "MultiState"
 ]
