@@ -47,12 +47,32 @@ class Hub:
         use_ssl: bool,
         installation_id: str | None = None,
         model_name: str | None = None,
-        serial: str = "noserial",
+        serial: str | None = "noserial",
+        topic_prefix: str | None = None,
     ) -> None:
         """Initialize."""
-        _LOGGER.debug(
-            "Initializing Hub(host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s)",
-            host, port, username, use_ssl, installation_id, model_name
+        # Parameter validation
+        if not isinstance(host, str) or not host:
+            raise ValueError("host must be a non-empty string")
+        if not isinstance(port, int) or not (0 < port < 65536):
+            raise ValueError("port must be an integer between 1 and 65535")
+        if username is not None and not isinstance(username, str):
+            raise TypeError(f"username must be a string or None, got type={type(username).__name__}, value={username!r}")
+        if password is not None and not isinstance(password, str):
+            raise TypeError(f"password must be a string or None, got type={type(password).__name__}, value={password!r}")
+        if not isinstance(use_ssl, bool):
+            raise TypeError(f"use_ssl must be a boolean, got type={type(use_ssl).__name__}, value={use_ssl!r}")
+        if installation_id is not None and not isinstance(installation_id, str):
+            raise TypeError(f"installation_id must be a string or None, got type={type(installation_id).__name__}, value={installation_id!r}")
+        if model_name is not None and not isinstance(model_name, str):
+            raise TypeError(f"model_name must be a string or None, got type={type(model_name).__name__}, value={model_name!r}")
+        if serial is not None and not isinstance(serial, str):
+            raise TypeError(f"serial must be a string or None, got type={type(serial).__name__}, value={serial!r}")
+        if topic_prefix is not None and not isinstance(topic_prefix, str):
+            raise TypeError(f"topic_prefix must be a string or None, got type={type(topic_prefix).__name__}, value={topic_prefix!r}")
+        _LOGGER.info(
+            "Initializing Hub(host=%s, port=%d, username=%s, use_ssl=%s, installation_id=%s, model_name=%s, topic_prefix=%s)",
+            host, port, username, use_ssl, installation_id, model_name, topic_prefix
         )
         self._model_name = model_name
         self.host = host
@@ -63,6 +83,7 @@ class Hub:
         self._client = None
         self.port = port
         self._installation_id = installation_id
+        self._topic_prefix = topic_prefix
         self.logger = logging.getLogger(__name__)
         self._devices: dict[str, Device] = {}
         self._first_refresh_event: asyncio.Event = asyncio.Event()
@@ -72,14 +93,38 @@ class Hub:
         self._connected_event = asyncio.Event()
         self._connected_failed = False
         # Replace all {placeholder} patterns with + for MQTT wildcards
-        self.topic_map = {Hub._remove_placeholders(desc.topic): desc for desc in topics}
-        self.fallback_map = {Hub._remove_placeholders(desc.topic.rsplit('/', 1)[0] + '/' + desc.is_adjustable_suffix): desc for desc in topics if desc.is_adjustable_suffix}
+        expanded_topics = Hub.expand_topic_list(topics)
+        def merge_is_adjustable_suffix(desc: TopicDescriptor) -> str:
+            """Merge the topic with its adjustable suffix."""
+            assert desc.is_adjustable_suffix is not None
+            return Hub._remove_placeholders_map(desc.topic.rsplit('/', 1)[0] + '/' + desc.is_adjustable_suffix)
+
+        # Helper to build a map where duplicate keys accumulate values in a list
+        def build_multi_map(items, key_func):
+            result = {}
+            for item in items:
+                key = key_func(item)
+                if key in result:
+                    existing = result[key]
+                    existing.append(item)
+                else:
+                    result[key] = [item]
+            return result
+
+        self.topic_map = build_multi_map(expanded_topics, lambda desc: Hub._remove_placeholders_map(desc.topic))
+        self.fallback_map = build_multi_map(
+            [desc for desc in expanded_topics if desc.is_adjustable_suffix],
+            lambda desc: Hub._remove_placeholders_map(merge_is_adjustable_suffix(desc))
+        )
+        subscription_list1 = [Hub._remove_placeholders(topic.topic) for topic in expanded_topics]
+        subscription_list2 = [Hub._remove_placeholders(merge_is_adjustable_suffix(desc)) for desc in expanded_topics if desc.is_adjustable_suffix]
+        self._subscription_list = subscription_list1 + subscription_list2
+        self._client = MQTTClient(callback_api_version=CallbackAPIVersion.VERSION2)
         _LOGGER.info("Hub initialized")
 
     async def connect(self) -> None:
         """Connect to the hub."""
         _LOGGER.info("Connecting to MQTT broker at %s:%d", self.host, self.port)
-        self._client = MQTTClient(callback_api_version=CallbackAPIVersion.VERSION2)
         
         if self.username is not None:
             _LOGGER.info("Setting auth credentials for user: %s", self.username)
@@ -120,7 +165,7 @@ class Hub:
             await self._wait_for_first_refresh()
             _LOGGER.info("Devices and metrics initialized. Found %d devices", len(self._devices))
         except Exception as exc:
-            _LOGGER.error("Failed to connect to MQTT broker: %s", exc)
+            _LOGGER.error("Failed to connect to MQTT broker: %s", exc, exc_info=True)
             raise CannotConnectError(f"Failed to connect to MQTT broker: {exc}") from exc
 
     def _on_log(self, client: MQTTClient, userdata: Any, level:int, buf:str) -> None:
@@ -163,10 +208,13 @@ class Hub:
         payload = message.payload
         _LOGGER.debug("Message received: topic=%s, payload=%s", topic, payload)
         
+        # Remove topic prefix before processing
+        topic = self._remove_topic_prefix(topic)
+        
         if "full_publish_completed" in topic:
             _LOGGER.info("Full publish completed, unsubscribing from notification")
             if self._client is not None and self._client.is_connected():
-                self._client.unsubscribe("N/+/full_publish_completed")
+                self._unsubscribe("N/+/full_publish_completed")
             self._loop.call_soon_threadsafe(self._first_refresh_event.set)
             return
 
@@ -194,18 +242,25 @@ class Hub:
             return
 
         fallback_to_metric_topic: bool = False
-        desc = self.topic_map.get(parsed_topic.wildcards_with_device_type)
-        if desc is None:
-            desc = self.topic_map.get(parsed_topic.wildcards_without_device_type)
-        if desc is None:
-            desc = self.fallback_map.get(parsed_topic.wildcards_with_device_type)
+        desc_list = self.topic_map.get(parsed_topic.wildcards_with_device_type)
+        if desc_list is None:
+            desc_list = self.topic_map.get(parsed_topic.wildcards_without_device_type)
+        if desc_list is None:
+            desc_list = self.fallback_map.get(parsed_topic.wildcards_with_device_type)
             fallback_to_metric_topic = True
-        if desc is None:
-            desc = self.fallback_map.get(parsed_topic.wildcards_without_device_type)
+        if desc_list is None:
+            desc_list = self.fallback_map.get(parsed_topic.wildcards_without_device_type)
             fallback_to_metric_topic = True
-        if desc is None:
+        if desc_list is None:
             _LOGGER.debug("Ignoring message - no descriptor found for topic: %s", topic)
             return
+        if len(desc_list) == 1:
+            desc = desc_list[0]
+        else:
+            desc = parsed_topic.match_from_list(desc_list)
+            if desc is None:
+                _LOGGER.debug("Ignoring message - no matching descriptor found for list of topic: %s", topic)
+                return
 
         device = self._get_or_create_device(parsed_topic, desc)
         device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload.decode(), self._loop, self)
@@ -239,7 +294,8 @@ class Hub:
 
     def publish(self, topic: str, value: PayloadType) -> None:
         assert self._client is not None
-        self._client.publish(topic, value)
+        prefixed_topic = self._add_topic_prefix(topic)
+        self._client.publish(prefixed_topic, value)
 
     async def _keep_alive_loop(self) -> None:
         """Run keep_alive every 30 seconds."""
@@ -283,7 +339,7 @@ class Hub:
         assert self._client is not None
         self._first_refresh_event.clear()
         self._client.on_message = self._on_snapshot_message
-        self._client.subscribe("#")
+        self._subscribe("#")
         _LOGGER.info("Subscribed to all topics for snapshot")
         await self._keep_alive()
         await self._wait_for_first_refresh()
@@ -303,14 +359,15 @@ class Hub:
     ) -> None:
         """Handle snapshot messages synchronously."""
         try:
-            if "full_publish_completed" in message.topic:
+            topic = self._remove_topic_prefix(message.topic)
+            if "full_publish_completed" in topic:
                 _LOGGER.info("Full publish completed, unsubscribing from notification")
                 if self._client is not None:
-                    self._client.unsubscribe("N/+/full_publish_completed")
+                    self._unsubscribe("N/+/full_publish_completed")
                 self._loop.call_soon_threadsafe(self._first_refresh_event.set)
                 return
 
-            topic_parts = message.topic.split("/")
+            topic_parts = topic.split("/")
             value = json.loads(message.payload.decode())
             self._set_nested_dict_value(self._snapshot, topic_parts, value)
         except Exception as exc:
@@ -324,7 +381,8 @@ class Hub:
     ) -> None:
         """Handle installation ID messages synchronously."""
         try:
-            topic_parts = message.topic.split("/")
+            topic = self._remove_topic_prefix(message.topic)
+            topic_parts = topic.split("/")
             if len(topic_parts) == 5 and topic_parts[2:5] == ["system", "0", "Serial"]:
                 payload_json = json.loads(message.payload.decode())
                 self._installation_id = payload_json.get("value")
@@ -344,14 +402,14 @@ class Hub:
             _LOGGER.error("Cannot read installation ID - client not connected")
             raise NotConnectedError
 
-        self._client.subscribe(TOPIC_INSTALLATION_ID)
+        self._subscribe(TOPIC_INSTALLATION_ID)
         try:
             await asyncio.wait_for(self._installation_id_event.wait(), timeout=60)
         except asyncio.TimeoutError:
             _LOGGER.error("Timeout waiting for installation ID")
             raise
         if self._client is not None:
-            self._client.unsubscribe(TOPIC_INSTALLATION_ID)
+            self._unsubscribe(TOPIC_INSTALLATION_ID)
         _LOGGER.info("Installation ID read successfully: %s", self.installation_id)
         return str(self.installation_id)
 
@@ -359,6 +417,44 @@ class Hub:
     def _remove_placeholders(topic: str) -> str:
         return re.sub(r'\{[^}]+\}', '+', topic)
 
+    @staticmethod
+    def _remove_placeholders_map(topic: str) -> str:
+        topic_parts = topic.split("/")
+        for i, part in enumerate(topic_parts):
+            if part == "{phase}":
+                topic_parts[i] = "##phase##"
+            elif part.isdigit() or (part.startswith("{") and part.endswith("}")):
+                topic_parts[i] = "##num##"
+        return "/".join(topic_parts)
+
+    def _add_topic_prefix(self, topic: str) -> str:
+        """Add the topic prefix to a topic if configured."""
+        if self._topic_prefix is None:
+            return topic
+        return f"{self._topic_prefix}/{topic}"
+
+    def _remove_topic_prefix(self, topic: str) -> str:
+        """Remove the topic prefix from a topic if configured."""
+        if self._topic_prefix is None:
+            return topic
+        if topic.startswith(f"{self._topic_prefix}/"):
+            return topic[len(self._topic_prefix) + 1:]
+        return topic
+
+    def _subscribe(self, topic: str) -> None:
+        """Subscribe to a topic with automatic prefix handling."""
+        assert self._client is not None
+        prefixed_topic = self._add_topic_prefix(topic)
+        _LOGGER.debug("Subscribing to: %s", prefixed_topic)
+        self._client.subscribe(prefixed_topic)
+
+    def _unsubscribe(self, topic: str) -> None:
+        """Unsubscribe from a topic with automatic prefix handling."""
+        assert self._client is not None
+        prefixed_topic = self._add_topic_prefix(topic)
+        self._client.unsubscribe(prefixed_topic)
+        _LOGGER.debug("Unsubscribed from: %s", prefixed_topic)
+
     def _setup_subscriptions(self) -> None:
         """Subscribe to list of topics."""
         _LOGGER.info("Setting up MQTT subscriptions")
@@ -366,14 +462,10 @@ class Hub:
         if not self._client.is_connected():
             raise NotConnectedError
         #topic_list = [(topic, 0) for topic in topic_map]
-        for topic in self.topic_map.keys():
-            self._client.subscribe(topic)
-            _LOGGER.debug("Subscribed to: %s", topic)
-        for topic in self.fallback_map.keys():
-            self._client.subscribe(topic)
-            _LOGGER.debug("Subscribed to fallback topic: %s", topic)
+        for topic in self._subscription_list:
+            self._subscribe(topic)
 
-        self._client.subscribe("N/+/full_publish_completed")
+        self._subscribe("N/+/full_publish_completed")
         _LOGGER.info("Subscribed to full_publish_completed notification")
 
     async def _wait_for_connect(self) -> None:
@@ -445,6 +537,11 @@ class Hub:
         """Return the model name."""
         return self._model_name
 
+    @property
+    def topic_prefix(self) -> str | None:
+        """Return the topic prefix."""
+        return self._topic_prefix
+
     @property
     def connected(self) -> bool:
         """Return if connected."""
@@ -452,13 +549,37 @@ class Hub:
             return False
         return self._client.is_connected()
 
-
     def on_connect_fail(self, client: MQTTClient, userdata: Any) -> None:
         """Handle connection failure callback."""
         _LOGGER.error("Connection to MQTT broker failed")
         self._connected_failed = True
         self._loop.call_soon_threadsafe(self._connected_event.set)
 
+    @staticmethod
+    def expand_topic_list(topic_list: list[TopicDescriptor]) -> list[TopicDescriptor]:
+        """
+        Expands TopicDescriptors with placeholders like {output(1-4)} into multiple descriptors.
+        """
+        import re
+        expanded = []
+        pattern = re.compile(r"\{([a-zA-Z0-9_]+)\((\d+)-(\d+)\)\}")
+        for td in topic_list:
+            matches = list(pattern.finditer(td.topic))
+            if matches:
+                # For each placeholder, expand all combinations
+                # Only support one placeholder per field for now
+                match = matches[0]
+                key, start, end = match.group(1), int(match.group(2)), int(match.group(3))
+                for i in range(start, end+1):
+                    new_kwargs = td.__dict__.copy()
+                    new_kwargs['topic'] = pattern.sub(str(i), td.topic)
+                    new_kwargs['key_values'] = {key: str(i)}
+                    expanded.append(TopicDescriptor(**new_kwargs))
+            else:
+                expanded.append(td)
+        return expanded
+
+
 
 class CannotConnectError(Exception):
     """Error to indicate we cannot connect."""
