@@ -26,13 +26,15 @@ class Metric:
             unique_id, descriptor.metric_type, descriptor.metric_nature
         )
         assert descriptor.name is not None, "name must be set for metric"
+        assert parsed_topic.device_type is not None, "device_type must be set for metric"
+        assert parsed_topic.device_type != DeviceType.UNKNOWN, "device_type must not be UNKNOWN for metric"
+        
         self._descriptor = descriptor
         self._unique_id = unique_id
         self._value = None
-        self._phase = parsed_topic.phase
         self._short_id = short_id
-        self._name = parsed_topic.get_name(descriptor)
-        self._key_values: dict[str, str] = parsed_topic.get_key_values(descriptor)
+        self._name = parsed_topic.name
+        self._key_values: dict[str, str] = parsed_topic.key_values
         self._on_update: Callable | None = None
         _LOGGER.debug("Metric %s initialized", repr(self))
 
@@ -45,7 +47,6 @@ class Metric:
             f"descriptor={self._descriptor}, "
             f"value={self.value}, "
             f"generic_short_id={self.generic_short_id}, "
-            f"phase={self.phase}, "
             f"device_type={self.device_type}, "
             f"short_id={self.short_id}, "
             f"name={self.name}, "
@@ -91,11 +92,6 @@ class Metric:
         """Returns the generic short id of the metric."""
         return self._descriptor.short_id
 
-    @property
-    def phase(self) -> str | None:
-        """Returns the phase of the metric if referring to a specific AC phase. None if not relevant"""
-        return self._phase
-
     @property
     def unit_of_measurement(self) -> str | None:
         """Returns the unit of measurement of the metric."""
@@ -127,7 +123,7 @@ class Metric:
         return self._descriptor.precision
 
     @property
-    def min_value(self) -> int | None:
+    def min_value(self) -> int | RangeType | None:
         return self._descriptor.min
 
     @property
@@ -146,7 +142,7 @@ class Metric:
     @property
     def key_values(self) -> dict[str, str]:
         """Return the key_values dictionary as read-only."""
-        return self._key_values.copy()
+        return self._key_values
 
     @property
     def on_update(self) -> Callable | None:
